/*
	syslogdctl - syslog deamon controler
	Copyright (C) 2003 ONO,Masayuki.<mono101@skill-engineering.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "syslogdctl.h"

#include <qtabwidget.h>
#include <qtextstream.h>
#include <qstringlist.h>
#include <qlabel.h>
#include <qmultilineedit.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qcheckbox.h>
#include <qfile.h>
#include <qdir.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <qpe/global.h>
#include <qmessagebox.h>

syslogdctl::syslogdctl( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : syslogdctlbase( parent, name, modal, fl )
{
	setWState( WState_Reserved1 );

	UpdateStatus();

	AutoExecCheck->setChecked( QFile::exists( "/etc/rc.d/rc5.d/S02syslog" ) );

	QFile f( "/etc/syslog.conf" );
	if ( f.open( IO_ReadOnly ) )
	{
		QTextStream stream( &f );
		const QStringList& lines = QStringList::split( '\n', stream.read(), TRUE );
		f.close();
		for ( QStringList::ConstIterator it = lines.begin(); it != lines.end(); ++ it )
		{
			SyslogConfEdit->append( *it );
		}
	}

	m_pTimer = new QTimer();
	m_pTimer->start( 1500 );
	connect( m_pTimer, SIGNAL( timeout() ), this, SLOT( UpdateStatus() ) );
	connect( StartButton, SIGNAL( clicked() ), this, SLOT( DoStart() ) );
	connect( StopButton, SIGNAL( clicked() ), this, SLOT( DoStop() ) );
	connect( SaveButton, SIGNAL( clicked() ), this, SLOT( DoSave() ) );
}

syslogdctl::~syslogdctl()
{
	m_pTimer->stop();
	disconnect( m_pTimer, SIGNAL( timeout() ), this, SLOT( UpdateStatus() ) );
	delete m_pTimer;
}

void syslogdctl::UpdateStatus()
{
	StatusEdit->setText( QFile::exists( "/var/run/syslogd.pid" ) ? "ALIVE" : "DEAD" );
	StartButton->setText( QFile::exists( "/var/run/syslogd.pid" ) 
											? tr( "ReStart syslogd" ) 
											: tr( "Start syslogd" ) );
}

void syslogdctl::DoStart()
{
	int nRel = 0;
	if ( QFile::exists( "/var/run/syslogd.pid" ) )
	{
		nRel = system( "sudo /etc/rc.d/init.d/syslog restart" );
	}
	else
	{
		nRel = system( "sudo /etc/rc.d/init.d/syslog start" );
	}

	if ( nRel != 0 )
	{
		QMessageBox::warning( this, "syslogd Controler", "/etc/rc.d/init.d/syslog\nExecute ERROR" );
	}
	else
	{
		QMessageBox::information( this, "syslogd Controler",
									"syslogd was (re)started" );
	}
}

void syslogdctl::DoSave()
{
	if ( SyslogConfEdit->edited() )
	{
		int nRel = system( "/bin/cp /etc/syslog.conf /etc/syslog.conf.bak" );
		if ( nRel != 0 )
		{
			QMessageBox::warning( this, "syslogd Controler",
						"/etc/syslog.conf\nBackup ERROR" );
			return;
		}
		QString sText = SyslogConfEdit->text();
		QFile file_out( "/etc/syslog.conf" );
		if ( file_out.open( IO_Truncate | IO_WriteOnly | IO_Raw ) )
		{
			QTextStream stream_out( &file_out );
			stream_out.writeRawBytes( sText.latin1(), sText.length() );
			file_out.close();
			QMessageBox::information( this, "syslogd Controler",
										"/etc/syslog.conf was saved" );
			SyslogConfEdit->clear();
			QFile file_in( "/etc/syslog.conf" );
			if ( file_in.open( IO_ReadOnly ) )
			{
				QTextStream stream_in( &file_in );
				const QStringList& lines = QStringList::split( '\n', stream_in.read(), TRUE );
				file_in.close();
				for ( QStringList::ConstIterator it = lines.begin(); it != lines.end(); ++ it )
				{
					SyslogConfEdit->append( *it );
				}
			}
		}
	}
}

void syslogdctl::DoStop()
{
	if ( QFile::exists( "/var/run/syslogd.pid" ) )
	{
		int nRel = 0;
		nRel = system( "sudo /etc/rc.d/init.d/syslog stop" );
		if ( nRel != 0 )
		{
			QMessageBox::warning( this, "syslogd Controler", "/etc/rc.d/init.d/syslog\nExecute ERROR" );
		}
		else
		{
			QMessageBox::information( this, "syslogd Controler",
										"syslogd was stopped" );
		}
	}
}

void syslogdctl::closeEvent( QCloseEvent* e )
{
	e->accept();
}

void syslogdctl::makeS02syslog()
{
	int nRel = system( "/bin/cp /etc/rc.d/rc5.d/_S02syslog /etc/rc.d/rc5.d/S02syslog" );
	if ( nRel != 0 )
	{
		QMessageBox::warning( this, "syslogd Controler", "cp Execute ERROR" );
	}
	else
	{
		QMessageBox::information( this, "syslogd Controler",
									"/etc/rc.d/rc5.d/S02syslog\nwas made as copy of _S02syslog" );
	}
}

void syslogdctl::delS02syslog()
{
	int nRel = system( "/bin/rm /etc/rc.d/rc5.d/S02syslog" );
	if ( nRel != 0 )
	{
		QMessageBox::warning( this, "syslogd Controler", "rm Execute ERROR" );
	}
	else
	{
		QMessageBox::information( this, "syslogd Controler",
									"/etc/rc.d/rc5.d/S02syslog\nwas removed" );
	}
}

void syslogdctl::accept()
{
	if ( AutoExecCheck->isChecked() )
	{
		if ( ! QFile::exists( "/etc/rc.d/rc5.d/S02syslog" ) )
		{
			makeS02syslog();
		}
	}
	else
	{
		if ( QFile::exists( "/etc/rc.d/rc5.d/S02syslog" ) )
		{
			delS02syslog();
		}
	}
	close();
}

void syslogdctl::reject()
{
	close();
}

